
/* 
 *
 *    v2.0 
 *
 *    Copyright (c) 2002, Geoff Ingram (geoff@dbcool.com) - Proton Technology Ltd
 *
 *    orabm_serverside_stress.sql: Oracle package for CPU stress test benchmark
 *
 *    You may distribute orabm_serverside_stress.sql under the terms of either the 
 *    widely available GNU General Public License or the Artistic License 
 *    with the exception that it cannot be placed on a CD-ROM or similar media 
 *    for commercial distribution without the prior approval of the author.
 *
 */   


create or replace PROCEDURE ORABM.SERVER_C_SP_Order_Status_ID (
  ware_id IN INTEGER,
  dist_id IN INTEGER,
  cust_id IN INTEGER
   ) IS

  TYPE rowidarray IS TABLE OF ROWID INDEX BY BINARY_INTEGER;
  cust_rowid			ROWID;
  ol				BINARY_INTEGER;
  c_num			BINARY_INTEGER;
  row_id			rowidarray;
  l_rows integer;
  l_where integer;


  -- move this from arg list
  bylastname INTEGER;
  cust_last customer.c_last%type;
  cust_first customer.c_first%type;
  cust_middle customer.c_middle%type;
  cust_balance customer.c_balance%type;
  ord_id orders.o_id%type;
  ord_entry_d orders.o_entry_d%type;
  ord_carrier_id orders.o_carrier_id%type;
  ord_ol_cnt orders.o_ol_cnt%type;

  CURSOR mo_cur IS
  SELECT o_id, o_entry_d, o_carrier_id, o_ol_cnt
  FROM Orders
  WHERE o_d_id = dist_id AND o_w_id = ware_id AND o_c_id = cust_id
  ORDER BY o_w_id, o_d_id, o_c_id, o_id DESC;

  BEGIN
    LOOP
      BEGIN
        SELECT c_balance, c_first, c_middle, c_last
        INTO cust_balance, cust_first, cust_middle, cust_last
        FROM Customer WHERE c_id = cust_id AND c_d_id = dist_id AND c_w_id = ware_id;
        OPEN mo_cur;
        FETCH mo_cur INTO ord_id, ord_entry_d, ord_carrier_id, ord_ol_cnt;
      
        CLOSE mo_cur;
        EXIT;
     END;
   END
  LOOP;

  l_rows:=0;

  FOR REC IN (select
		ol_supply_w_id,
		ol_i_id,
		ol_quantity,
		ol_amount,
		ol_delivery_d
	from
		Order_Line
	where
		ol_o_id = ord_id
		and ol_d_id = dist_id
		and ol_w_id = ware_id) loop
            l_rows:=l_rows+1;
  end loop;

EXCEPTION when others then
   if (l_where=1) then
	raise_application_error(-20002,'SERVER_C_SP_Order_Status_ID '||l_where||':'||sqlerrm
                       ||' c='||cust_id||' d='||dist_id||' w='||ware_id);
   end if;
END;
/



create or replace PROCEDURE ORABM.SERVER_C_SP_Order_Status_Name (
  ware_id			INTEGER,
  dist_id			INTEGER,
  cust_last		IN OUT VARCHAR2)

IS TYPE rowidarray IS TABLE OF ROWID INDEX BY BINARY_INTEGER;
cust_id integer;
cust_rowid			ROWID;
ol				BINARY_INTEGER;
c_num				BINARY_INTEGER;
row_id				rowidarray;
l_rows integer;
CURSOR c_cur IS

SELECT rowid
FROM Customer
WHERE c_d_id = dist_id 	AND c_w_id = ware_id 	AND c_last = cust_last
ORDER BY c_w_id, c_d_id, c_last, c_first;




CURSOR mo_cur IS
SELECT o_id, o_entry_d, o_carrier_id, o_ol_cnt
FROM Orders
WHERE o_d_id = dist_id AND o_w_id = ware_id AND o_c_id = cust_id
ORDER BY o_w_id, o_d_id, o_c_id, o_id DESC;

 -- move this from arg list
  bylastname INTEGER;
    cust_first customer.c_first%type;
  cust_middle customer.c_middle%type;
  cust_balance customer.c_balance%type;
  ord_id orders.o_id%type;
  ord_entry_d orders.o_entry_d%type;
  ord_carrier_id orders.o_carrier_id%type;
  ord_ol_cnt orders.o_ol_cnt%type;


BEGIN
  LOOP
    BEGIN
      c_num := 0;
      FOR c_id_rec IN c_cur LOOP
        c_num := c_num +1;
        row_id(c_num) := c_id_rec.rowid;
      END LOOP;

      cust_rowid := row_id((c_num+1)/2);

      SELECT c_id, c_balance, c_first, c_middle, c_last
      INTO cust_id, cust_balance, cust_first, cust_middle, cust_last
      FROM Customer WHERE rowid = cust_rowid;

      OPEN mo_cur;
      FETCH mo_cur INTO ord_id, ord_entry_d, ord_carrier_id, ord_ol_cnt;
      CLOSE mo_cur;
      EXIT;
   END;
  END LOOP;

  l_rows:=0;
  FOR REC IN (select
		ol_supply_w_id,
		ol_i_id,
		ol_quantity,
		ol_amount,
		ol_delivery_d
	from
		Order_Line
	where
		ol_o_id = ord_id
		and ol_d_id = dist_id
		and ol_w_id = ware_id) loop
            l_rows:=l_rows+1;
     end loop;

END;
/



create or replace PROCEDURE ORABM.SERVER_C_SP_Stock_Level (ware_id INTEGER, 	
                                   dist_id INTEGER,
                                   threshold INTEGER,
                                   low_stock OUT INTEGER ) IS 
BEGIN
  LOOP
    BEGIN
    SELECT COUNT(DISTINCT s_i_id) INTO low_stock
      FROM Order_Line, Stock, District
      WHERE d_id = dist_id AND d_w_id = ware_id  AND d_id = ol_d_id  AND d_w_id = ol_w_id
      AND ol_i_id = s_i_id  AND ol_w_id = s_w_id AND s_quantity < threshold
      AND ol_o_id BETWEEN (d_next_o_id-20) AND (d_next_o_id-1);
    EXIT;
    END;
  END LOOP;
END;
/


create or replace Procedure ORABM.ORABM_SERVERSIDE_STRESS(
     p_trans in integer,
     p_result in out varchar2) AS
   l_split integer;
   l_warehouse_id integer;
   l_district_id integer;
   l_threshold integer;
   l_low_stock integer;
   l_random integer;
   l_cust_id integer;
   l_last customer.c_last%type;
   p_first customer.c_first%type;
   p_last customer.c_last%type;
   p_middle customer.c_middle%type;
   p_balance customer.c_balance%type;
   l_where integer;
   l_trans_done integer;
   l_start date;
   l_end   date;
   l_secs number;
   l_tps number;
   p_stock number:=0;
   p_order_by_id number:=0;
   p_order_by_name number:=0;
   l_global varchar2(80);
    -- ignore 10% or trans at start and end
   l_trans_ramp_start integer;
   l_trans_ramp_end integer;
   l_trans_sampled integer;
   cursor c_cust(p_c_id integer) is
       select c_last from customer where c_id=p_c_id;
BEGIN
    p_stock:=0;
    p_order_by_id:=0;
    p_order_by_name:=0;
    l_trans_done:=0;
    l_warehouse_id:=1;

    select global_name into l_global from global_name;
    l_trans_ramp_start:=p_trans/10;
    l_trans_ramp_end:=p_trans*9/10;
    loop
      -- start the clock after ramp start reached
      if l_trans_done = l_trans_ramp_start then
         l_start := sysdate;
      end if;


       if l_trans_done=p_trans then

         l_secs:= (l_end-l_start)*24*60*60;
         l_trans_sampled:=p_stock+p_order_by_name+p_order_by_id;
         l_tps := l_trans_sampled/l_secs;
         p_result:= l_global||' txn(all)='||p_trans||
            ' xn(sam)='||l_trans_sampled||
            ' t(sam)='||round(l_secs)||' tps='||round(l_tps)||
            ' sl='||p_stock||'('||round(100*(p_stock/l_trans_sampled),1)||'%)'||
            ' on='||p_order_by_name||'('||round(100*(p_order_by_name/l_trans_sampled),1)||'%)'||
            ' oi='||p_order_by_id||'('||round(100*(p_order_by_id/l_trans_sampled),1)||'%)'||
            ' end='||to_char(sysdate,'DDMMYY-HH24:MI:SS');
         return;
       end if;

    l_split :=DBMS_RANDOM.VALUE(LOW=>1,HIGH=>100);
    l_district_id:=round(Dbms_Random.Value(LOW=>1,HIGH=>10));

    if l_split <= 50 then -- Stock-Level
      if l_trans_done >= l_trans_ramp_start
      and l_trans_done < l_trans_ramp_end-1 then
         p_stock:=p_stock+1;
      end if;

      l_threshold:= round(Dbms_Random.Value(LOW=>10,HIGH=>20));

      l_where:=1;

      Server_C_Sp_Stock_Level(ware_id=>l_warehouse_id,
	                       dist_id=>l_district_id,
	                       threshold=>l_threshold,
                              low_stock=>l_low_stock);
    else -- Order-Status
      -- random customer id
      l_cust_id :=round(DBMS_RANDOM.VALUE(LOW=>1,HIGH=>3000));
      l_split := Dbms_Random.Value(LOW=>1,HIGH=>100);
      l_where:=2;
      if l_split <= 40 then -- Order-Status by id
        if l_trans_done >= l_trans_ramp_start
        and l_trans_done < l_trans_ramp_end-1 then
          p_order_by_id:=p_order_by_id+1;
        end if;

        Server_C_Sp_Order_Status_id(
	      ware_id=>l_warehouse_id,
	      dist_id=>l_district_id,
             cust_id=>l_cust_id);

      else -- Order-Status by name
        if l_trans_done >= l_trans_ramp_start
        and l_trans_done < l_trans_ramp_end-1 then
          p_order_by_name:=p_order_by_name+1;
        end if;

        open c_cust(l_cust_id);
        fetch c_cust into l_last;
        Server_C_Sp_Order_Status_Name(
	       ware_id=>l_warehouse_id,
	       dist_id=>l_district_id,
	       cust_last=>l_last);

        close c_cust;

      end if;

  end if;
  l_trans_done:=l_trans_done+1;
  -- stop the clock if ramp end reached
  if l_trans_done = l_trans_ramp_end-1 then
    l_end := sysdate;
  end if;

end loop;
EXCEPTION
   when others then
      raise_application_error(-20001,
         'orabm_serverside_stress:' ||l_where||':'||sqlerrm);
END ORABM_SERVERSIDE_STRESS;
/

exit
