/* 
 *
 *    v2.0 
 *
 *    Copyright (c) 2002, Geoff Ingram (geoff@dbcool.com) - Proton Technology Ltd
 *
 *    orabm.c, Oracle CPU stress test benchmark.
 *
 *    You may distribute orabm.c under the terms of either the 
 *    widely available GNU General Public License or the Artistic License 
 *    with the exception that it cannot be placed on a CD-ROM or similar media 
 *    for commercial distribution without the prior approval of the author.
 *
 */   
#include <unistd.h>
#include <string.h>
#include <pthread.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <oci.h>  

text *sql = (text *) 
         "BEGIN orabm_serverside_stress(:p_in_txn,:p_out_result); END;";

text *traceon=(text *)"alter session set sql_trace=true";

static void checkerr(/*_ OCIError *errhp, sword status _*/);  
sb4 status;
void *benchmark(void*);

FILE *log;

char tns[128]; 
char orabm_trace[128]; 

int main(argc, argv)
int argc;
char *argv[];
{
   int txn;
   int i;
   pthread_t threads[1024];
   char conn[256];
   char logname[256];
   char sessions[256];
   int  s;
   char *sp;
   char *str;
   int iterations[256]; /* different sess values from csv input e.g. 1,2,4,8,16 */
   int iteration; /* current iteration */
   int sess;      /* concurrent Oracle sessions in current iteration */
   time_t now;
   long x;

   if (argc <3 || argc > 4)
   {
     printf("order <sessions> <transactions> [TNS]\n");
     exit(1);
   }

   strcpy(sessions,argv[1]); /* csv list */
   txn=atoi(argv[2]);

   if (argc == 4)
   {
      sprintf(logname,"bm.%s.log",argv[3]);
      strcpy(tns,argv[3]);
      sprintf(conn,"TNS=%s",tns); 
   }	  
   else
   {
      char sid[32];
      if (getenv("ORACLE_SID"))
        sprintf(sid,"%s",getenv("ORACLE_SID"));
      else
      {
        printf("ORACLE_SID not set\n");
        exit(1);
      }

      sprintf(logname,"orabm.%s.log",sid);
      sprintf(conn,"ORACLE_SID=%s",sid);
   }

   if (getenv ("ORACLE_TRACE"))
      sprintf (orabm_trace, "%s", getenv ("ORACLE_TRACE"));

   if ((log=fopen(logname,"a+"))==(FILE*)NULL)
   {
     printf("orabm: failed to open log %s (%)\n",logname,strerror(errno));
     exit(1);
   }

   sp=sessions;

   i=0;
   while ((str=strtok(sp,",")) != NULL) 
   {
      /*printf("iteration[%d]=%s\n",i,str);*/
      iterations[i++]=atoi(str);
      sp=NULL; /* strtok() requires NULL after 1st call */
   }

   /* 
      i is now total iterations 
      eg for input 1,2,4 => 
      3 iterations: 1 concurrent session - 2 concurrent sessions - 4 concurrent sessions
   */

   for (iteration=0;iteration<i;iteration++)
   {
      time(&now);
      sess=iterations[iteration]; /* sessions for this iteration */
      printf("---begin sess=%d txn=%d %s %s",sess,txn,conn,ctime(&now));
      fprintf(log,"---begin sess=%d txn=%d %s %s",sess,txn,conn,ctime(&now));
      fflush(log);

      /* create 1 thread per Oracle session */
      for (s=0;s<sess;s++)
      {
         if (pthread_create(&threads[s], NULL, benchmark, (void *)&txn))
         {
           printf("pthread_create() error=%d\n",strerror(errno));
         }
      }
   
      for (s=0;s<sess;s++)
      {
         if (pthread_join(threads[s],NULL))
         {
            printf("pthread_join() error=%d\n",strerror(errno));
         }
      }
      time(&now);
      printf("---end - %s",ctime(&now));
      fprintf(log,"---end - %s",ctime(&now));
      fflush(log);
   }

}


void *benchmark(txn)
void *txn;
{
  sword status;
  sword    p_in_txn,p_stock,p_order_by_id,p_order_by_name;
  char     p_out_result[256];
  short    p_out_result_ind;
  dvoid    *tmp;
  OCIEnv *envhp;
  OCIError *errhp;  
  char *username="ORABM";
  char *password="ORABM";
  OCISession *authp = (OCISession *)NULL; 
  OCIServer *srvhp;
  OCISvcCtx *svchp;
  OCIStmt   *stmthp;
  OCIStmt   *stmthp2;
  OCIBind  *bnd1p = (OCIBind *) 0;             /* the first bind handle */
  OCIBind  *bnd2p = (OCIBind *) 0;             /* the second bind handle */

  p_in_txn=*((sword*)txn);

  OCIEnvCreate (&envhp, OCI_THREADED, (dvoid *)0,  NULL,
                NULL, NULL, 0, (dvoid *)0);       

/*  printf("envhp=%lx\n",envhp); */

  (void) OCIHandleAlloc( (dvoid *) envhp, (dvoid **) &errhp, OCI_HTYPE_ERROR,
                   (size_t) 0, (dvoid **) 0);

/*   printf("errhp=%lx\n",errhp); */

  /* server contexts */
  (void) OCIHandleAlloc( (dvoid *) envhp, (dvoid **) &srvhp, OCI_HTYPE_SERVER,
                   (size_t) 0, (dvoid **) 0);

/*   printf("srvhp=%lx\n",srvhp); */

  (void) OCIHandleAlloc( (dvoid *) envhp, (dvoid **) &svchp, OCI_HTYPE_SVCCTX,
                   (size_t) 0, (dvoid **) 0);

   if (tns[0]==0) /* use tns */
   {
     status=OCIServerAttach( srvhp, errhp, (text *)"", strlen(""), 0);
   }
   else
   {
      status = OCIServerAttach(srvhp, errhp, (text *)tns,
      (sb4)strlen((char *)tns), OCI_DEFAULT);
   }

   if (status != OCI_SUCCESS)
   {
       printf("OCIServerAttach failed\n"); exit(1);
   }

  /* set attribute server context in the service context */
  (void) OCIAttrSet( (dvoid *) svchp, OCI_HTYPE_SVCCTX, (dvoid *)srvhp,
                     (ub4) 0, OCI_ATTR_SERVER, (OCIError *) errhp);

  (void) OCIHandleAlloc((dvoid *) envhp, (dvoid **)&authp,
                        (ub4) OCI_HTYPE_SESSION, (size_t) 0, (dvoid **) 0);

/*   printf("authp=%lx\n",authp); */
  (void) OCIAttrSet((dvoid *) authp, (ub4) OCI_HTYPE_SESSION,
                 (dvoid *) username, (ub4) strlen((char *)username),
                 (ub4) OCI_ATTR_USERNAME, errhp);

  (void) OCIAttrSet((dvoid *) authp, (ub4) OCI_HTYPE_SESSION,
                 (dvoid *) password, (ub4) strlen((char *)password),
                 (ub4) OCI_ATTR_PASSWORD, errhp);

  checkerr(errhp, OCISessionBegin ( svchp,  errhp, authp, OCI_CRED_RDBMS,
                          (ub4) OCI_DEFAULT));

/*   printf("OCISessionBegin\n"); */

  (void) OCIAttrSet((dvoid *) svchp, (ub4) OCI_HTYPE_SVCCTX,
                   (dvoid *) authp, (ub4) 0,
                   (ub4) OCI_ATTR_SESSION, errhp);

  /* allocate a statement handle */
  checkerr(errhp, OCIHandleAlloc( (dvoid *) envhp, (dvoid **) &stmthp,
           OCI_HTYPE_STMT, 100, (dvoid **) &tmp));

  if (orabm_trace[0] == 'Y')
  {
     checkerr(errhp, OCIHandleAlloc( (dvoid *) envhp, (dvoid **) &stmthp2,
           OCI_HTYPE_STMT, (size_t) 0, (dvoid **) 0));

	 checkerr(errhp, OCIStmtPrepare(stmthp2, errhp, traceon,
                                (ub4) strlen((char *) traceon),
                                (ub4) OCI_NTV_SYNTAX, (ub4) OCI_DEFAULT));

     checkerr(errhp,OCIStmtExecute(svchp, stmthp2, errhp, (ub4) 1, (ub4) 0,
               (CONST OCISnapshot *) NULL, (OCISnapshot *) NULL, OCI_DEFAULT));
  }

  /* prepare the statement request, passing the PL/SQL text block */
  checkerr(errhp, OCIStmtPrepare(stmthp, errhp, (text *) sql, (ub4) 
      strlen(sql), OCI_NTV_SYNTAX, OCI_DEFAULT));

      /* bind each of the placeholders to a program variable */
   checkerr( errhp, OCIBindByName(stmthp, &bnd1p, errhp, (text *) ":p_in_txn",
             -1, (ub1 *) &p_in_txn,
            (sword) sizeof(p_in_txn), SQLT_INT, (dvoid *) 0,
             (ub2 *) 0, (ub2) 0, (ub4) 0, (ub4 *) 0, OCI_DEFAULT));

      /* remember that PL/SQL OUT variable are bound, not defined */
    checkerr(errhp,OCIBindByName(stmthp, &bnd2p, errhp, (text *) ":p_out_result",
             -1, (dvoid *) p_out_result,
             sizeof(p_out_result), SQLT_STR, (dvoid *) &p_out_result_ind,
             (ub2 *) 0, (ub2 *) 0, (ub4) 0, (ub4 *) 0, OCI_DEFAULT));

/*   printf("before SQL...\n"); */

  /* execute PL/SQL block*/
  checkerr(errhp, OCIStmtExecute(svchp, stmthp, errhp, (ub4) 1, (ub4) 0, 
      (OCISnapshot *) NULL, (OCISnapshot *) NULL, OCI_DEFAULT));

  printf("%s\n",p_out_result);
  fprintf(log,"%s\n",p_out_result);
  fflush(log);

  /* close out connection to the database */

   OCIHandleFree(( dvoid * ) stmthp, OCI_HTYPE_STMT );

   checkerr( errhp, OCISessionEnd( svchp, errhp, authp, OCI_DEFAULT ));

   checkerr( errhp, OCIHandleFree(( dvoid * ) authp, OCI_HTYPE_SESSION ));

   checkerr( errhp, OCIHandleFree(( dvoid * ) svchp, OCI_HTYPE_SVCCTX ));

   checkerr( errhp, OCIServerDetach( srvhp, errhp, OCI_DEFAULT ));

   checkerr( errhp, OCIHandleFree(( dvoid * ) srvhp, OCI_HTYPE_SERVER ));

   checkerr( errhp, OCIHandleFree(( dvoid * ) errhp, OCI_HTYPE_ERROR ));

   checkerr( errhp, OCIHandleFree(( dvoid * ) envhp, OCI_HTYPE_ENV ));
}

void checkerr(errhp, status)
OCIError *errhp;
sword status;
{
  text errbuf[512];
  sb4 errcode = 0;

  switch (status)
  {
  case OCI_SUCCESS:
    break;
  case OCI_SUCCESS_WITH_INFO:
    (void) printf("Error - OCI_SUCCESS_WITH_INFO\n");
    break;
  case OCI_NEED_DATA:
    (void) printf("Error - OCI_NEED_DATA\n");
    break;
  case OCI_NO_DATA:
    (void) printf("Error - OCI_NO_DATA\n");
    break;
  case OCI_ERROR:
    (void) OCIErrorGet((dvoid *)errhp, (ub4) 1, (text *) NULL, &errcode,
                        errbuf, (ub4) sizeof(errbuf), OCI_HTYPE_ERROR);
    (void) printf("Error - %.*s\n", 512, errbuf);
    break;
  case OCI_INVALID_HANDLE:
    (void) printf("Error - OCI_INVALID_HANDLE\n");
    break;
  case OCI_STILL_EXECUTING:
    (void) printf("Error - OCI_STILL_EXECUTE\n");
    break;
  case OCI_CONTINUE:
    (void) printf("Error - OCI_CONTINUE\n");
    break;
  default:
    break;
  }
}
